import copy

import pytest
from icupy import U_ICU_VERSION_MAJOR_NUM

if U_ICU_VERSION_MAJOR_NUM < 60:
    pytest.skip("ICU4C<60", allow_module_level=True)

from icupy import (
    CurrencyUnit, DecimalFormatSymbols, ErrorCode, Format, Formattable,
    INT32_MAX, Locale, MeasureUnit, UCurrencyUsage, UErrorCode,
    UNumberDecimalSeparatorDisplay, UNumberFormatRoundingMode,
    UNumberSignDisplay, UNumberUnitWidth, UParseError, UnicodeString,
    UnicodeStringAppendable,
)
from icupy.number import (
    CurrencyPrecision, FormattedNumber, FractionPrecision, IncrementPrecision,
    IntegerWidth, LocalizedNumberFormatter, Notation, NumberFormatter,
    Precision, ScientificNotation, UnlocalizedNumberFormatter,
)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_formatted_number_62():
    fmt = (NumberFormatter.with_()
           .notation(Notation.compact_short())
           .unit(MeasureUnit.get_kelvin())
           .locale("en-US"))
    num = fmt.format_double(65000)
    assert isinstance(num, FormattedNumber)

    # Appendable &icu::number::FormattedNumber::appendTo(
    #       Appendable &appendable,
    #       UErrorCode &status
    # )
    s = UnicodeString()
    appendable = UnicodeStringAppendable(s)
    result = num.append_to(appendable)
    assert isinstance(result, UnicodeStringAppendable)
    assert id(result) == id(appendable)
    assert s == "65K K"

    # UnicodeString icu::number::FormattedNumber::toString(UErrorCode &status)
    result = num.to_string()
    assert isinstance(result, UnicodeString)
    assert result == "65K K"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_formatted_number_64():
    from icupy import (
        ConstrainedFieldPosition, FormattedValue, UFieldCategory,
        UNumberFormatFields,
    )

    assert issubclass(FormattedNumber, FormattedValue)

    fmt = (NumberFormatter.with_()
           .notation(Notation.compact_short())
           .unit(MeasureUnit.get_kelvin())
           .locale("en-US"))
    num = fmt.format_double(65000)
    assert isinstance(num, FormattedNumber)

    # UBool icu::number::FormattedNumber::nextPosition(
    #       ConstrainedFieldPosition &cfpos,
    #       UErrorCode &status
    # )
    cfpos = ConstrainedFieldPosition()
    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_INTEGER_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (0, 2)

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_COMPACT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (2, 3)

    assert num.next_position(cfpos)
    assert cfpos.get_category() == UFieldCategory.UFIELD_CATEGORY_NUMBER
    assert cfpos.get_field() == UNumberFormatFields.UNUM_MEASURE_UNIT_FIELD
    assert (cfpos.get_start(), cfpos.get_limit()) == (4, 5)

    assert not num.next_position(cfpos)

    # UnicodeString icu::number::FormattedNumber::toTempString(
    #       UErrorCode &status
    # )
    result = num.to_temp_string()
    assert isinstance(result, UnicodeString)
    assert result == "65K K"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 65, reason="ICU4C<65")
def test_formatted_number_65():
    fmt = (NumberFormatter.with_()
           .notation(Notation.compact_short())
           .unit(MeasureUnit.get_kelvin())
           .locale("en-US"))
    num = fmt.format_double(65000)
    assert isinstance(num, FormattedNumber)

    # template<typename StringClass>
    # StringClass icu::number::FormattedNumber::toDecimalNumber(
    #       UErrorCode &status
    # )
    result = num.to_decimal_number()
    assert isinstance(result, str)
    assert result == "65"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_formatted_number_68():
    fmt = (NumberFormatter.with_()
           .usage("person")
           .unit(MeasureUnit.get_kilogram())
           .locale("en-GB"))
    num = fmt.format_double(80)
    assert isinstance(num, FormattedNumber)

    # MeasureUnit icu::number::FormattedNumber::getOutputUnit(
    #       UErrorCode &status
    # )
    unit = num.get_output_unit()
    assert isinstance(unit, MeasureUnit)
    assert unit.get_identifier() == "stone-and-pound"


def test_integer_width():
    # icu::number::IntegerWidth
    assert isinstance(IntegerWidth.zero_fill_to(3),
                      IntegerWidth)
    assert isinstance(IntegerWidth.zero_fill_to(3).truncate_at(3),
                      IntegerWidth)


def test_localized_number_formatter_60():
    fmt = NumberFormatter.with_locale(Locale.get_us())
    assert isinstance(fmt, LocalizedNumberFormatter)

    # [2]
    # icu::number::LocalizedNumberFormatter::LocalizedNumberFormatter(
    #       const LocalizedNumberFormatter &other
    # )
    fmt2 = LocalizedNumberFormatter(fmt)

    # FormattedNumber icu::number::LocalizedNumberFormatter::formatDecimal(
    #       StringPiece value,
    #       UErrorCode &status
    # )
    num = fmt2.format_decimal("9.87E-3")
    assert isinstance(num, FormattedNumber)

    # FormattedNumber icu::number::LocalizedNumberFormatter::formatDouble(
    #       double value,
    #       UErrorCode &status
    # )
    num = fmt2.format_double(9.87E-3)
    assert isinstance(num, FormattedNumber)

    # FormattedNumber icu::number::LocalizedNumberFormatter::formatInt(
    #       int64_t value,
    #       UErrorCode &status
    # )
    num = fmt2.format_int(INT32_MAX + 1)
    assert isinstance(num, FormattedNumber)

    # UBool icu::number::NumberFormatterSettings<Derived>::copyErrorTo(
    #       UErrorCode &outErrorCode
    # )
    out_error_code = ErrorCode()
    assert not fmt2.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::decimal(
    #       UNumberDecimalSeparatorDisplay style
    # )
    result = fmt2.decimal(
        UNumberDecimalSeparatorDisplay.UNUM_DECIMAL_SEPARATOR_AUTO)
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::integerWidth(
    #       const IntegerWidth &style
    # )
    result = fmt2.integer_width(IntegerWidth.zero_fill_to(2))
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::notation(
    #       const Notation &notation
    # )
    result = fmt2.notation(Notation.compact_long())
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::sign(
    #       UNumberSignDisplay style
    # )
    result = fmt.sign(UNumberSignDisplay.UNUM_SIGN_AUTO)
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::symbols(
    #       const DecimalFormatSymbols &symbols
    # )
    result = fmt.symbols(DecimalFormatSymbols(Locale.get_us()))
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unit(
    #       const icu::MeasureUnit &unit
    # )
    result = fmt.unit(MeasureUnit.create_meter_per_second())
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unitWidth(
    #       UNumberUnitWidth width
    # )
    result = fmt.unit_width(UNumberUnitWidth.UNUM_UNIT_WIDTH_FULL_NAME)
    assert isinstance(result, LocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 61, reason="ICU4C<61")
def test_localized_number_formatter_61():
    from icupy import UNumberGroupingStrategy

    fmt = NumberFormatter.with_locale(Locale.get_us())
    assert isinstance(fmt, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::grouping(
    #       UNumberGroupingStrategy strategy
    # )
    result = fmt.grouping(UNumberGroupingStrategy.UNUM_GROUPING_MIN2)
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::perUnit(
    #       const icu::MeasureUnit &perUnit
    # )
    result = fmt.per_unit(MeasureUnit.get_meter())
    assert isinstance(result, LocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_localized_number_formatter_62():
    from icupy.number import Scale

    # [1]
    # icu::number::LocalizedNumberFormatter::LocalizedNumberFormatter()
    fmt = LocalizedNumberFormatter()

    # Format *icu::number::LocalizedNumberFormatter::toFormat(
    #       UErrorCode &status
    # )
    fmt2 = (fmt.notation(Notation.engineering())
            .unit(MeasureUnit.get_kelvin())
            .to_format())
    assert isinstance(fmt2, Format)

    obj = Formattable(65000)
    append_to = UnicodeString()
    fmt2.format(obj, append_to)
    assert append_to == "65E3 K"

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::precision(
    #       const Precision &precision
    # )
    result = fmt.precision(Precision.fixed_fraction(2))
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::roundingMode(
    #       UNumberFormatRoundingMode roundingMode
    # )
    result = fmt.rounding_mode(UNumberFormatRoundingMode.UNUM_ROUND_HALFEVEN)
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::scale(
    #       const Scale &scale
    # )
    result = fmt.scale(Scale.power_of_ten(2))
    assert isinstance(result, LocalizedNumberFormatter)

    # template<typename Derived>
    # UnicodeString icu::number::NumberFormatterSettings<Derived>::toSkeleton(
    #       UErrorCode &status
    # )
    result = fmt.unit(MeasureUnit.get_meter_per_second()).to_skeleton()
    assert isinstance(result, UnicodeString)
    assert result in ("unit/meter-per-second",
                      "measure-unit/speed-meter-per-second")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_localized_number_formatter_64():
    fmt = (NumberFormatter.with_locale(Locale.get_us())
           .notation(Notation.engineering()))
    assert isinstance(fmt, LocalizedNumberFormatter)
    assert fmt.format_double(0.8765).to_string() == "876.5E-3"

    # template<typename Derived>
    # LocalPointer<Derived>
    # icu::number::NumberFormatterSettings<Derived>::clone()
    fmt2 = fmt.clone()
    assert isinstance(fmt2, LocalizedNumberFormatter)
    assert fmt2.format_double(0.8765).to_string() == "876.5E-3"

    fmt3 = copy.copy(fmt)
    assert fmt3.format_double(0.8765).to_string() == "876.5E-3"

    fmt4 = copy.deepcopy(fmt)
    assert fmt4.format_double(0.8765).to_string() == "876.5E-3"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_localized_number_formatter_68():
    fmt = NumberFormatter.with_locale(Locale.get_us())
    assert isinstance(fmt, LocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::usage(
    #       StringPiece usage
    # )
    result = fmt.usage("person")
    assert isinstance(result, LocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_no_unit():
    from icupy import NoUnit

    fmt = NumberFormatter.with_locale(Locale.get_us())

    unit = NoUnit.percent()
    assert isinstance(unit, MeasureUnit)
    assert fmt.unit(unit).format_int(1).to_string() == "1%"

    unit = NoUnit.permille()
    assert isinstance(unit, MeasureUnit)
    assert fmt.unit(unit).format_int(1).to_string() == "1\u2030"  # 1‰


def test_number_formatter_60():
    # static UnlocalizedNumberFormatter icu::number::NumberFormatter::with()
    fmt = NumberFormatter.with_()
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    # static LocalizedNumberFormatter icu::number::NumberFormatter::withLocale(
    #       const Locale &locale
    # )
    fmt = NumberFormatter.with_locale(Locale("en_US"))
    assert isinstance(fmt, LocalizedNumberFormatter)

    fmt = NumberFormatter.with_locale("en_US")
    assert isinstance(fmt, LocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_number_formatter_62():
    # [1]
    # static UnlocalizedNumberFormatter
    # icu::number::NumberFormatter::forSkeleton(
    #       const UnicodeString &skeleton,
    #       UErrorCode &status
    # )
    fmt = NumberFormatter.for_skeleton(
        UnicodeString("measure-unit/speed-meter-per-second"))
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    fmt = NumberFormatter.for_skeleton(
        "measure-unit/speed-meter-per-second")
    assert isinstance(fmt, UnlocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_number_formatter_64():
    # [2]
    # static UnlocalizedNumberFormatter
    # icu::number::NumberFormatter::forSkeleton(
    #       const UnicodeString &skeleton,
    #       UParseError &perror,
    #       UErrorCode &status
    # )
    perror = UParseError()
    fmt = NumberFormatter.for_skeleton(
        UnicodeString("measure-unit/speed-meter-per-second"),
        perror)
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    perror = UParseError()
    fmt = NumberFormatter.for_skeleton(
        "measure-unit/speed-meter-per-second",
        perror)
    assert isinstance(fmt, UnlocalizedNumberFormatter)


def test_notation():
    # icu::number::Notation
    assert isinstance(Notation.compact_long(), Notation)
    assert isinstance(Notation.compact_short(), Notation)
    assert isinstance(Notation.engineering(), ScientificNotation)
    assert isinstance(Notation.scientific(), ScientificNotation)
    assert isinstance(Notation.simple(), Notation)

    # icu::number::ScientificNotation
    assert issubclass(ScientificNotation, Notation)

    assert isinstance(
        (Notation.engineering()
         .with_exponent_sign_display(UNumberSignDisplay.UNUM_SIGN_ALWAYS)),
        ScientificNotation)
    assert isinstance(
        Notation.engineering().with_min_exponent_digits(2),
        ScientificNotation)


def test_precision_60():
    # icu::number::Precision
    assert isinstance(Precision.currency(UCurrencyUsage.UCURR_USAGE_STANDARD),
                      CurrencyPrecision)
    assert isinstance(Precision.fixed_fraction(3), FractionPrecision)
    assert isinstance(Precision.increment(0.5), IncrementPrecision)
    assert isinstance(Precision.integer(), FractionPrecision)
    assert isinstance(Precision.max_fraction(3), FractionPrecision)
    assert isinstance(Precision.min_fraction(1), FractionPrecision)
    assert isinstance(Precision.min_max_fraction(1, 3), FractionPrecision)
    assert isinstance(Precision.unlimited(), Precision)

    # icu::number::CurrencyPrecision
    assert issubclass(CurrencyPrecision, Precision)

    assert isinstance(
        (Precision.currency(UCurrencyUsage.UCURR_USAGE_STANDARD)
         .with_currency(CurrencyUnit("USD"))),
        Precision)

    # icu::number::FractionPrecision
    assert issubclass(FractionPrecision, Precision)

    assert isinstance(Precision.fixed_fraction(2).with_max_digits(2),
                      Precision)
    assert isinstance(Precision.fixed_fraction(2).with_min_digits(3),
                      Precision)

    # icu::number::IncrementPrecision
    assert issubclass(IncrementPrecision, Precision)

    assert isinstance(Precision.increment(0.5).with_min_fraction(1),
                      Precision)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_precision_62():
    # icu::number::Precision
    assert isinstance(Precision.fixed_significant_digits(3), Precision)
    assert isinstance(Precision.max_significant_digits(4), Precision)
    assert isinstance(Precision.min_max_significant_digits(3, 4), Precision)
    assert isinstance(Precision.min_significant_digits(3), Precision)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 69, reason="ICU4C<69")
def test_precision_69():
    from icupy import UNumberRoundingPriority, UNumberTrailingZeroDisplay

    # icu::number::Precision
    assert isinstance(
        (Precision.fixed_fraction(2)
            .trailing_zero_display(
            UNumberTrailingZeroDisplay.UNUM_TRAILING_ZERO_AUTO)),
        Precision)

    # icu::number::FractionPrecision
    assert isinstance(
        (Precision.fixed_fraction(1)
            .with_significant_digits(
            3,
            3,
            UNumberRoundingPriority.UNUM_ROUNDING_PRIORITY_RELAXED)),
        Precision)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_scale():
    from icupy.number import Scale

    # icu::number::Scale
    assert isinstance(Scale.by_decimal("5.2"), Scale)
    assert isinstance(Scale.by_double(5.2), Scale)
    assert isinstance(Scale.by_double_and_power_of_ten(5.2, 3), Scale)
    assert isinstance(Scale.none(), Scale)
    assert isinstance(Scale.power_of_ten(2), Scale)


def test_unlocalized_number_formatter_60():
    fmt = NumberFormatter.with_()
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    # [2]
    # icu::number::UnlocalizedNumberFormatter::UnlocalizedNumberFormatter(
    #       const UnlocalizedNumberFormatter &other
    # )
    fmt2 = UnlocalizedNumberFormatter(fmt)

    # LocalizedNumberFormatter icu::number::UnlocalizedNumberFormatter::locale(
    #       const icu::Locale &locale
    # )
    result = fmt2.locale(Locale.get_us())
    assert isinstance(result, LocalizedNumberFormatter)

    result = fmt2.locale("en-US")
    assert isinstance(result, LocalizedNumberFormatter)

    # UBool icu::number::NumberFormatterSettings<Derived>::copyErrorTo(
    #       UErrorCode &outErrorCode
    # )
    out_error_code = ErrorCode()
    assert not fmt2.copy_error_to(out_error_code)
    assert out_error_code.get() == UErrorCode.U_ZERO_ERROR

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::decimal(
    #       UNumberDecimalSeparatorDisplay style
    # )
    result = fmt2.decimal(
        UNumberDecimalSeparatorDisplay.UNUM_DECIMAL_SEPARATOR_AUTO)
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::integerWidth(
    #       const IntegerWidth &style
    # )
    result = fmt2.integer_width(IntegerWidth.zero_fill_to(2))
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::notation(
    #       const Notation &notation
    # )
    result = fmt2.notation(Notation.compact_long())
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::sign(
    #       UNumberSignDisplay style
    # )
    result = fmt.sign(UNumberSignDisplay.UNUM_SIGN_AUTO)
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::symbols(
    #       const DecimalFormatSymbols &symbols
    # )
    result = fmt.symbols(DecimalFormatSymbols(Locale.get_us()))
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unit(
    #       const icu::MeasureUnit &unit
    # )
    result = fmt.unit(MeasureUnit.create_meter_per_second())
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::unitWidth(
    #       UNumberUnitWidth width
    # )
    result = fmt.unit_width(UNumberUnitWidth.UNUM_UNIT_WIDTH_FULL_NAME)
    assert isinstance(result, UnlocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 61, reason="ICU4C<61")
def test_unlocalized_number_formatter_61():
    from icupy import UNumberGroupingStrategy

    fmt = NumberFormatter.with_()
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::grouping(
    #       UNumberGroupingStrategy strategy
    # )
    result = fmt.grouping(UNumberGroupingStrategy.UNUM_GROUPING_MIN2)
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::perUnit(
    #       const icu::MeasureUnit &perUnit
    # )
    result = fmt.per_unit(MeasureUnit.get_meter())
    assert isinstance(result, UnlocalizedNumberFormatter)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_unlocalized_number_formatter_62():
    from icupy.number import Scale

    # [1]
    # icu::number::UnlocalizedNumberFormatter::UnlocalizedNumberFormatter()
    fmt = UnlocalizedNumberFormatter()

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::precision(
    #       const Precision &precision
    # )
    result = fmt.precision(Precision.fixed_fraction(2))
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::roundingMode(
    #       UNumberFormatRoundingMode roundingMode
    # )
    result = fmt.rounding_mode(UNumberFormatRoundingMode.UNUM_ROUND_HALFEVEN)
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::scale(
    #       const Scale &scale
    # )
    result = fmt.scale(Scale.power_of_ten(2))
    assert isinstance(result, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # UnicodeString icu::number::NumberFormatterSettings<Derived>::toSkeleton(
    #       UErrorCode &status
    # )
    result = fmt.unit(MeasureUnit.get_meter_per_second()).to_skeleton()
    assert isinstance(result, UnicodeString)
    assert result in ("unit/meter-per-second",
                      "measure-unit/speed-meter-per-second")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_unlocalized_number_formatter_64():
    fmt = NumberFormatter.with_().notation(Notation.engineering())
    assert isinstance(fmt, UnlocalizedNumberFormatter)
    assert fmt.locale("en-US").format_double(0.8765).to_string() == "876.5E-3"

    # template<typename Derived>
    # LocalPointer<Derived>
    # icu::number::NumberFormatterSettings<Derived>::clone()
    fmt2 = fmt.clone()
    assert isinstance(fmt2, UnlocalizedNumberFormatter)
    assert fmt2.locale("en-US").format_double(0.8765).to_string() == "876.5E-3"

    fmt3 = copy.copy(fmt)
    assert fmt3.locale("en-US").format_double(0.8765).to_string() == "876.5E-3"

    fmt4 = copy.deepcopy(fmt)
    assert fmt4.locale("en-US").format_double(0.8765).to_string() == "876.5E-3"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 68, reason="ICU4C<68")
def test_unlocalized_number_formatter_68():
    fmt = NumberFormatter.with_()
    assert isinstance(fmt, UnlocalizedNumberFormatter)

    # template<typename Derived>
    # Derived icu::number::NumberFormatterSettings<Derived>::usage(
    #       StringPiece usage
    # )
    result = fmt.usage("person")
    assert isinstance(result, UnlocalizedNumberFormatter)



import copy

import pytest
from icupy import (
    DecimalFormat, FieldPosition, FieldPositionIterator, Formattable,
    ICUError, Locale, MessageFormat, ParsePosition, SimpleDateFormat,
    StringEnumeration, TimeZone, UErrorCode, UMessagePatternApostropheMode,
    UParseError, U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


def test_api():
    # static UnicodeString icu::MessageFormat::autoQuoteApostrophe(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    result = MessageFormat.auto_quote_apostrophe(UnicodeString("'"))
    assert isinstance(result, UnicodeString)
    assert result == "''"

    result = MessageFormat.auto_quote_apostrophe("'")
    assert isinstance(result, UnicodeString)
    assert result == "''"

    pattern = UnicodeString(
        "At {1,time,::jmm} on {1,date,::dMMMM}, "
        "there was {2} on planet {0,number}.",
    )
    fmt = MessageFormat(pattern, Locale.get_us())

    # UMessagePatternApostropheMode icu::MessageFormat::getApostropheMode()
    assert (fmt.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL)

    # Format *icu::MessageFormat::getFormat(
    #       const UnicodeString &formatName,
    #       UErrorCode &status
    # )
    result = fmt.get_format(UnicodeString("0"))
    assert isinstance(result, DecimalFormat)

    result = fmt.get_format(UnicodeString("1"))
    assert isinstance(result, SimpleDateFormat)

    result = fmt.get_format(UnicodeString("2"))
    assert result is None

    result = fmt.get_format("0")
    assert isinstance(result, DecimalFormat)

    result = fmt.get_format("1")
    assert isinstance(result, SimpleDateFormat)

    result = fmt.get_format("2")
    assert result is None

    # StringEnumeration *icu::MessageFormat::getFormatNames(UErrorCode &status)
    it = fmt.get_format_names()
    assert isinstance(it, StringEnumeration)
    assert list(it) == ["1", "1", "2", "0"]

    # const Format **icu::MessageFormat::getFormats(int32_t &count)
    formats = fmt.get_formats()
    assert isinstance(formats, list)
    assert len(formats) == 4
    assert isinstance(formats[0], SimpleDateFormat)
    assert isinstance(formats[1], SimpleDateFormat)
    assert formats[2] is None
    assert isinstance(formats[3], DecimalFormat)

    # const Locale &icu::MessageFormat::getLocale(void)
    result = fmt.get_locale()
    assert isinstance(result, Locale)
    assert result == Locale("en", "US")

    # void icu::MessageFormat::setLocale(const Locale &theLocale)
    fmt.set_locale(Locale.get_france())
    assert fmt.get_locale() == Locale("fr", "FR")

    fmt.set_locale("en-US")
    assert fmt.get_locale() == Locale("en", "US")

    # UnicodeString &icu::MessageFormat::toPattern(UnicodeString &appendTo)
    append_to = UnicodeString()
    result = fmt.to_pattern(append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == pattern

    # UBool icu::MessageFormat::usesNamedArguments()
    assert not fmt.uses_named_arguments()

    # UBool icu::MessageFormat::operator==(const Format &other)
    fmt1 = MessageFormat("{0} {1}", Locale.get_us())
    fmt2 = MessageFormat("{0} {1}", Locale.get_us())
    fmt3 = MessageFormat("{0} {1}", Locale.get_france())
    assert fmt1 == fmt2
    assert not (fmt1 == fmt3)
    assert not (fmt2 == fmt3)


def test_apply_pattern():
    fmt1 = MessageFormat("{0} {1}")
    fmt2 = fmt1.clone()
    assert fmt1 == fmt2

    # [1]
    # void icu::MessageFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt1.apply_pattern(UnicodeString("{1} {0}"))
    assert fmt1 != fmt2

    fmt2.apply_pattern("{1} {0}")
    assert fmt1 == fmt2

    # [2]
    # void icu::MessageFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UMessagePatternApostropheMode aposMode,
    #       UParseError *parseError,
    #       UErrorCode &status
    # )
    pattern = UnicodeString("ab{0,choice,0#1'2''3'''4''''.}yz")
    parse_error = UParseError()
    fmt1.apply_pattern(
        pattern,
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED,
        parse_error)
    assert (fmt1.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED)
    assert fmt1 != fmt2

    parse_error = UParseError()
    fmt2.apply_pattern(
        str(pattern),
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED,
        parse_error)
    assert (fmt2.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_REQUIRED)
    assert fmt1 == fmt2

    fmt1.apply_pattern(
        pattern,
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL,
        None)
    assert (fmt1.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL)
    assert fmt1 != fmt2

    fmt2.apply_pattern(
        str(pattern),
        UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL,
        None)
    assert (fmt2.get_apostrophe_mode()
            == UMessagePatternApostropheMode.UMSGPAT_APOS_DOUBLE_OPTIONAL)
    assert fmt1 == fmt2

    # [3]
    # void icu::MessageFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    parse_error = UParseError()
    fmt1.apply_pattern(UnicodeString("{1} {0}"), parse_error)
    assert fmt1 != fmt2

    parse_error = UParseError()
    fmt2.apply_pattern("{1} {0}", parse_error)
    assert fmt1 == fmt2


def test_clone():
    fmt1 = MessageFormat("{0} {1}")

    # MessageFormat *icu::MessageFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, MessageFormat)
    assert fmt1 == fmt2

    fmt3 = copy.copy(fmt1)
    assert fmt1 == fmt3

    fmt4 = copy.deepcopy(fmt1)
    assert fmt1 == fmt4


def test_format():
    fmt = MessageFormat(
        "At {1,time,::jmm} on {1,date,::dMMMM}, "
        "there was {2} on planet {0,number}.",
        Locale.get_us())
    formats = fmt.get_formats()
    tz = TimeZone.get_gmt()
    formats[0].set_time_zone(tz)
    formats[1].set_time_zone(tz)
    date = 1637685775000.0  # 2021-11-23T16:42:55Z
    append_to = UnicodeString()

    # [1], [2]
    # UnicodeString &icu::MessageFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    obj = Formattable([
        Formattable(7),
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
    ])
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(obj, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [3]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    obj = Formattable([
        Formattable(7),
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
    ])
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(obj, append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    append_to.remove()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(obj, append_to, None)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [4]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    obj = Formattable([
        Formattable(7),
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
    ])
    append_to.remove()
    result = fmt.format(obj, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )

    # [5]
    # UnicodeString &icu::MessageFormat::format(
    #       const Formattable *source,
    #       int32_t count,
    #       UnicodeString &appendTo,
    #       FieldPosition &ignore,
    #       UErrorCode &status
    # )
    source = [
        Formattable(7),
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
    ]
    append_to.remove()
    ignore = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(source, len(source), append_to, ignore)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )

    append_to.remove()
    ignore = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(source, -1, append_to, ignore)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )

    # *NotImplemented*
    # [6]
    # static UnicodeString &icu::MessageFormat::format(
    #       const UnicodeString &pattern,
    #       const Formattable *arguments,
    #       int32_t count,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )

    # [7]
    # UnicodeString &icu::MessageFormat::format(
    #       const UnicodeString *argumentNames,
    #       const Formattable *arguments,
    #       int32_t count,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    argument_names = [
        UnicodeString("1"),
        UnicodeString("2"),
        UnicodeString("0"),
    ]
    arguments = [
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
        Formattable(7),
    ]
    append_to.remove()
    result = fmt.format(
        argument_names,
        arguments,
        len(argument_names),
        append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )

    append_to.remove()
    result = fmt.format(
        argument_names,
        arguments,
        -1,
        append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == (
        "At 4:42 PM on November 23, "
        "there was a disturbance in the Force on planet 7."
    )


def test_message_format():
    pattern = UnicodeString("{0} {1}")
    new_locale = Locale.get_us()

    # [1]
    # icu::MessageFormat::MessageFormat(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt1 = MessageFormat(pattern)

    fmt1a = MessageFormat(str(pattern))
    assert fmt1 == fmt1a

    # [2]
    # icu::MessageFormat::MessageFormat(
    #       const UnicodeString &pattern,
    #       const Locale &newLocale,
    #       UErrorCode &status
    # )
    fmt2 = MessageFormat(pattern, new_locale)

    fmt2a = MessageFormat(str(pattern), new_locale)
    assert fmt2 == fmt2a

    fmt2b = MessageFormat(pattern, "en-US")
    assert fmt2 == fmt2b

    fmt2c = MessageFormat(str(pattern), "en-US")
    assert fmt2 == fmt2c

    # [3]
    # icu::MessageFormat::MessageFormat(
    #       const UnicodeString &pattern,
    #       const Locale &newLocale,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    parse_error = UParseError()
    fmt3 = MessageFormat(pattern, new_locale, parse_error)

    parse_error = UParseError()
    fmt3a = MessageFormat(str(pattern), new_locale, parse_error)
    assert fmt3 == fmt3a

    parse_error = UParseError()
    fmt3b = MessageFormat(pattern, "en-US", parse_error)
    assert fmt3 == fmt3b

    parse_error = UParseError()
    fmt3c = MessageFormat(str(pattern), "en-US", parse_error)
    assert fmt3 == fmt3c

    # [4]
    # icu::MessageFormat::MessageFormat(const MessageFormat &)
    fmt4 = MessageFormat(fmt1)
    assert fmt1 == fmt4


def test_parse():
    # From icu/source/test/intltest/tmsgfmt.cpp
    fmt = MessageFormat("{0} =sep= {1}", Locale.get_us())
    s = "abc =sep= def"

    # [1]
    # Formattable *icu::MessageFormat::parse(
    #       const UnicodeString &source,
    #       int32_t &count,
    #       UErrorCode &status
    # )
    objs = fmt.parse(UnicodeString(s))
    assert isinstance(objs, list)
    assert len(objs) == 2
    assert all(isinstance(x, Formattable) for x in objs)
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    objs = fmt.parse(s)
    assert isinstance(objs, list)
    assert len(objs) == 2
    assert all(isinstance(x, Formattable) for x in objs)
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    # [2]
    # Formattable *icu::MessageFormat::parse(
    #       const UnicodeString &source,
    #       ParsePosition &pos,
    #       int32_t &count
    # )
    pos = ParsePosition()
    objs = fmt.parse(UnicodeString(s), pos)
    assert pos.get_index() > 0
    assert pos.get_error_index() == -1
    assert isinstance(objs, list)
    assert len(objs) == 2
    assert all(isinstance(x, Formattable) for x in objs)
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    pos = ParsePosition()
    objs = fmt.parse(s, pos)
    assert pos.get_index() > 0
    assert pos.get_error_index() == -1
    assert isinstance(objs, list)
    assert len(objs) == 2
    assert all(isinstance(x, Formattable) for x in objs)
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    # void icu::MessageFormat::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       ParsePosition &pos
    # )
    result = Formattable()
    pos = ParsePosition()
    fmt.parse_object(UnicodeString(s), result, pos)
    assert pos.get_index() > 0
    assert pos.get_error_index() == -1
    assert result.get_type() == Formattable.ARRAY
    objs = result.get_array()
    assert len(objs) == 2
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    result = Formattable()
    pos = ParsePosition()
    fmt.parse_object(s, result, pos)
    assert pos.get_index() > 0
    assert pos.get_error_index() == -1
    assert result.get_type() == Formattable.ARRAY
    objs = result.get_array()
    assert len(objs) == 2
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    # void icu::Format::parseObject(
    #       const UnicodeString &source,
    #       Formattable &result,
    #       UErrorCode &status
    # )
    result = Formattable()
    fmt.parse_object(UnicodeString(s), result)
    assert result.get_type() == Formattable.ARRAY
    objs = result.get_array()
    assert len(objs) == 2
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"

    result = Formattable()
    fmt.parse_object(s, result)
    assert result.get_type() == Formattable.ARRAY
    objs = result.get_array()
    assert len(objs) == 2
    assert objs[0].get_type() == Formattable.STRING
    assert objs[0].get_string() == "abc"
    assert objs[1].get_type() == Formattable.STRING
    assert objs[1].get_string() == "def"


def test_set_format():
    fmt = MessageFormat("{0,date} {1,time}", Locale.get_us())
    dtfmt0 = SimpleDateFormat("yyyy.MM.dd")
    dtfmt1 = SimpleDateFormat("HH:mm:ss z")
    date = 1637685775000.0  # 2021-11-23T16:42:55Z
    obj = Formattable([
        Formattable(date, Formattable.IS_DATE),
        Formattable(date, Formattable.IS_DATE),
    ])
    append_to = UnicodeString()

    # [1]
    # void icu::MessageFormat::setFormat(
    #       const UnicodeString &formatName,
    #       const Format &format,
    #       UErrorCode &status
    # )
    tz = TimeZone.get_gmt()
    dtfmt0.set_time_zone(tz)
    dtfmt1.set_time_zone(tz)

    fmt.set_format(UnicodeString("0"), dtfmt0)
    fmt.set_format("1", dtfmt1)

    append_to.remove()
    assert fmt.format(obj, append_to) == "2021.11.23 16:42:55 GMT"

    # [2]
    # void icu::MessageFormat::setFormat(
    #       int32_t formatNumber,
    #       const Format &format
    # )
    tz = TimeZone.create_time_zone("PST")
    dtfmt0.set_time_zone(tz)
    dtfmt1.set_time_zone(tz)

    fmt.set_format(0, dtfmt0)
    fmt.set_format(1, dtfmt1)

    append_to.remove()
    assert fmt.format(obj, append_to) in ("2021.11.23 08:42:55 GMT-8",
                                          "2021.11.23 08:42:55 PST")


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_set_format_62():
    from icupy.number import NumberFormatter

    fmt = MessageFormat("{0,number,currency}", Locale.get_us())
    obj = Formattable([
        Formattable(444444.55)
    ])
    append_to = UnicodeString()
    assert fmt.format(obj, append_to) == "$444,444.55"

    numfmt = (
        NumberFormatter.for_skeleton("currency/USD unit-width-iso-code")
            .locale(Locale.get_us())
            .to_format()
    )
    fmt.set_format(0, numfmt)
    append_to.remove()
    assert fmt.format(obj, append_to) == "USD\xa0444,444.55"


def test_set_formats():
    fmt = MessageFormat(
        "At {1,time,::jmm} on {1,date,::dMMMM}, "
        "there was {2} on planet {0,number}.",
        Locale.get_us())
    formats = fmt.get_formats()

    # void icu::MessageFormat::setFormats(
    #       const Format **newFormats,
    #       int32_t cnt
    # )
    new_formats = [
        SimpleDateFormat("HH:mm:ss z"),
        SimpleDateFormat("yyyy.MM.dd"),
        None,
        formats[3].clone(),
    ]
    tz = TimeZone.get_gmt()
    new_formats[0].set_time_zone(tz)
    new_formats[1].set_time_zone(tz)
    fmt.set_formats(new_formats, len(new_formats))

    date = 1637685775000.0  # 2021-11-23T16:42:55Z
    obj = Formattable([
        Formattable(7),
        Formattable(date, Formattable.IS_DATE),
        Formattable(UnicodeString("a disturbance in the Force")),
    ])
    append_to = UnicodeString()
    assert fmt.format(obj, append_to) == (
        "At 16:42:55 GMT on 2021.11.23, "
        "there was a disturbance in the Force on planet 7."
    )

    tz = TimeZone.create_time_zone("PST")
    new_formats[0].set_time_zone(tz)
    new_formats[1].set_time_zone(tz)
    fmt.set_formats(new_formats)  # cnt is optional

    append_to.remove()
    assert fmt.format(obj, append_to) in (
        "At 08:42:55 GMT-8 on 2021.11.23, "
        "there was a disturbance in the Force on planet 7.",
        "At 08:42:55 PST on 2021.11.23, "
        "there was a disturbance in the Force on planet 7.",
    )



import copy

import pytest
from icupy import (
    CurrencyPluralInfo, DecimalFormat, DecimalFormatSymbols, FieldPosition,
    FieldPositionIterator, Formattable, INT32_MAX, Locale, NumberFormat,
    ParsePosition, UParseError, U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


def test_api():
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # UBool icu::DecimalFormat::areSignificantDigitsUsed()
    assert not fmt.are_significant_digits_used()

    # void icu::DecimalFormat::setSignificantDigitsUsed(
    #       UBool useSignificantDigits
    # )
    fmt.set_significant_digits_used(True)
    assert fmt.are_significant_digits_used()

    # const CurrencyPluralInfo *icu::DecimalFormat::getCurrencyPluralInfo(void)
    result = fmt.get_currency_plural_info()
    assert result is None

    # void icu::DecimalFormat::setCurrencyPluralInfo(
    #       const CurrencyPluralInfo &info
    # )
    info = CurrencyPluralInfo(Locale.get_france())
    fmt.set_currency_plural_info(info)
    result = fmt.get_currency_plural_info()
    assert isinstance(result, CurrencyPluralInfo)
    assert result == info

    # const DecimalFormatSymbols *icu::DecimalFormat::getDecimalFormatSymbols()
    result = fmt.get_decimal_format_symbols()
    assert isinstance(result, DecimalFormatSymbols)
    assert result == symbols

    # void icu::DecimalFormat::setDecimalFormatSymbols(
    #       const DecimalFormatSymbols &symbols
    # )
    symbols2 = DecimalFormatSymbols(Locale.get_france())
    assert symbols != symbols2
    fmt.set_decimal_format_symbols(symbols2)
    assert fmt.get_decimal_format_symbols() == symbols2

    # int32_t icu::DecimalFormat::getFormatWidth(void)
    assert fmt.get_format_width() == -1

    # void icu::DecimalFormat::setFormatWidth(int32_t width)
    fmt.set_format_width(0)
    assert fmt.get_format_width() == 0

    # int32_t icu::DecimalFormat::getGroupingSize(void)
    assert fmt.get_grouping_size() == 0

    # void icu::DecimalFormat::setGroupingSize(int32_t newValue)
    fmt.set_grouping_size(3)
    assert fmt.get_grouping_size() == 3

    # int32_t icu::DecimalFormat::getMaximumSignificantDigits()
    assert fmt.get_maximum_significant_digits() == 6

    # void icu::DecimalFormat::setMaximumSignificantDigits(int32_t max)
    fmt.set_maximum_significant_digits(5)
    assert fmt.get_maximum_significant_digits() == 5

    # int8_t icu::DecimalFormat::getMinimumExponentDigits(void)
    assert fmt.get_minimum_exponent_digits() == -1

    # void icu::DecimalFormat::setMinimumExponentDigits(int8_t minExpDig)
    fmt.set_minimum_exponent_digits(1)
    assert fmt.get_minimum_exponent_digits() == 1

    # int32_t icu::DecimalFormat::getMinimumSignificantDigits()
    assert fmt.get_minimum_significant_digits() == 1

    # void icu::DecimalFormat::setMinimumSignificantDigits(int32_t min)
    fmt.set_minimum_significant_digits(2)
    assert fmt.get_minimum_significant_digits() == 2

    # int32_t icu::DecimalFormat::getMultiplier(void)
    assert fmt.get_multiplier() == 1

    # void icu::DecimalFormat::setMultiplier(int32_t newValue)
    fmt.set_multiplier(10)
    assert fmt.get_multiplier() == 10

    # UnicodeString &icu::DecimalFormat::getNegativePrefix(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = fmt.get_negative_prefix(result)
    assert isinstance(output, UnicodeString)
    assert id(result) == id(output)
    assert result == "-"

    # void icu::DecimalFormat::setNegativePrefix(const UnicodeString &newValue)
    fmt.set_negative_prefix(UnicodeString("\u2212"))
    fmt.set_negative_prefix("\u2212")
    assert fmt.get_negative_prefix(result) == "\u2212"  # "−"

    # UnicodeString &icu::DecimalFormat::getNegativeSuffix(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = fmt.get_negative_suffix(result)
    assert isinstance(output, UnicodeString)
    assert id(result) == id(output)
    assert result == " euros"

    # void icu::DecimalFormat::setNegativeSuffix(const UnicodeString &newValue)
    fmt.set_negative_suffix(UnicodeString("~"))
    fmt.set_negative_suffix("~")
    assert fmt.get_negative_suffix(result) == "~"

    # UnicodeString icu::DecimalFormat::getPadCharacterString()
    result = fmt.get_pad_character_string()
    assert isinstance(result, UnicodeString)
    assert result == "\x20"

    # void icu::DecimalFormat::setPadCharacter(const UnicodeString &padChar)
    fmt.set_pad_character(UnicodeString("_"))
    fmt.set_pad_character("_")
    result = fmt.get_pad_character_string()
    assert result == "_"

    # EPadPosition icu::DecimalFormat::getPadPosition(void)
    assert fmt.get_pad_position() == DecimalFormat.PAD_BEFORE_PREFIX

    # void icu::DecimalFormat::setPadPosition(EPadPosition padPos)
    fmt.set_pad_position(DecimalFormat.PAD_BEFORE_SUFFIX)
    assert fmt.get_pad_position() == DecimalFormat.PAD_BEFORE_SUFFIX

    # UnicodeString &icu::DecimalFormat::getPositivePrefix(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = fmt.get_positive_prefix(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert len(result) == 0

    # void icu::DecimalFormat::setPositivePrefix(const UnicodeString &newValue)
    fmt.set_positive_prefix(UnicodeString("+"))
    fmt.set_positive_prefix("+")
    assert fmt.get_positive_prefix(result) == "+"

    # UnicodeString &icu::DecimalFormat::getPositiveSuffix(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = fmt.get_positive_suffix(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert result == " euros"

    # void icu::DecimalFormat::setPositiveSuffix(const UnicodeString &newValue)
    fmt.set_positive_suffix(UnicodeString("_"))
    fmt.set_positive_suffix("_")
    assert fmt.get_positive_suffix(result) == "_"

    # double icu::DecimalFormat::getRoundingIncrement(void)
    assert fmt.get_rounding_increment() == 0.0

    # void icu::DecimalFormat::setRoundingIncrement(double newValue)
    fmt.set_rounding_increment(1.2)
    assert fmt.get_rounding_increment() == 1.2

    # ERoundingMode icu::DecimalFormat::getRoundingMode(void)
    assert fmt.get_rounding_mode() == DecimalFormat.ROUND_HALF_EVEN

    # void icu::DecimalFormat::setRoundingMode(ERoundingMode roundingMode)
    fmt.set_rounding_mode(DecimalFormat.ROUND_CEILING)
    assert fmt.get_rounding_mode() == DecimalFormat.ROUND_CEILING

    # int32_t icu::DecimalFormat::getSecondaryGroupingSize(void)
    assert fmt.get_secondary_grouping_size() == 0

    # void icu::DecimalFormat::setSecondaryGroupingSize(int32_t newValue)
    fmt.set_secondary_grouping_size(3)
    assert fmt.get_secondary_grouping_size() == 3

    # UBool icu::DecimalFormat::isDecimalSeparatorAlwaysShown(void)
    assert not fmt.is_decimal_separator_always_shown()

    # void icu::DecimalFormat::setDecimalSeparatorAlwaysShown(UBool newValue)
    fmt.set_decimal_separator_always_shown(True)
    assert fmt.is_decimal_separator_always_shown()

    # UBool icu::DecimalFormat::isExponentSignAlwaysShown(void)
    assert not fmt.is_exponent_sign_always_shown()

    # void icu::DecimalFormat::setExponentSignAlwaysShown(UBool expSignAlways)
    fmt.set_exponent_sign_always_shown(True)
    assert fmt.is_exponent_sign_always_shown()

    # UBool icu::DecimalFormat::isScientificNotation(void)
    assert fmt.is_scientific_notation()

    # void icu::DecimalFormat::setScientificNotation(UBool useScientific)
    fmt.set_scientific_notation(False)
    assert not fmt.is_scientific_notation()

    # UBool icu::DecimalFormat::operator==(const Format &other)
    fmt2 = DecimalFormat("0", DecimalFormatSymbols(Locale.get_us()))
    fmt3 = fmt2.clone()
    assert not (fmt == fmt2)
    assert not (fmt == fmt3)
    assert fmt2 == fmt3

    # void icu::DecimalFormat::setCurrency(
    #       const char16_t *theCurrency,
    #       UErrorCode &ec
    # )
    fmt.set_currency("EUR")

    # void icu::DecimalFormat::setLenient(UBool enable)
    fmt.set_lenient(True)

    # void icu::DecimalFormat::setMaximumFractionDigits(int32_t newValue)
    fmt.set_maximum_fraction_digits(4)

    # void icu::DecimalFormat::setMaximumIntegerDigits(int32_t newValue)
    fmt.set_maximum_integer_digits(4)

    # void icu::DecimalFormat::setMinimumFractionDigits(int32_t newValue)
    fmt.set_minimum_fraction_digits(0)

    # void icu::DecimalFormat::setMinimumIntegerDigits(int32_t newValue)
    fmt.set_minimum_integer_digits(0)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_api_49():
    from icupy import CurrencyAmount

    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # CurrencyAmount *icu::DecimalFormat::parseCurrency(
    #       const UnicodeString &text,
    #       ParsePosition &pos
    # )
    pos = ParsePosition()
    camt = fmt.parse_currency(UnicodeString("USD-1,234.00"), pos)
    assert pos.get_error_index() == -1
    assert isinstance(camt, CurrencyAmount)

    pos = ParsePosition()
    camt = fmt.parse_currency(UnicodeString("-1,234.00"), pos)
    assert pos.get_error_index() != -1
    assert camt is None

    pos = ParsePosition()
    camt = fmt.parse_currency("USD-1,234.00", pos)
    assert pos.get_error_index() == -1
    assert isinstance(camt, CurrencyAmount)

    pos = ParsePosition()
    camt = fmt.parse_currency("-1,234.00", pos)
    assert pos.get_error_index() != -1
    assert camt is None


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 51, reason="ICU4C<51")
def test_api_51():
    from icupy import UNumberFormatAttribute

    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # int32_t icu::DecimalFormat::getAttribute(
    #       UNumberFormatAttribute attr,
    #       UErrorCode &status
    # )
    assert fmt.get_attribute(UNumberFormatAttribute.UNUM_SCALE) == 0

    # DecimalFormat &icu::DecimalFormat::setAttribute(
    #       UNumberFormatAttribute attr,
    #       int32_t newValue,
    #       UErrorCode &status
    # )
    result = fmt.set_attribute(
        UNumberFormatAttribute.UNUM_SCALE,
        -2)
    assert isinstance(result, DecimalFormat)
    assert id(result) == id(fmt)
    assert fmt.get_attribute(UNumberFormatAttribute.UNUM_SCALE) == -2


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 53, reason="ICU4C<53")
def test_api_53():
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # void icu::DecimalFormat::setGroupingUsed(UBool newValue)
    fmt.set_grouping_used(True)

    # void icu::DecimalFormat::setParseIntegerOnly(UBool value)
    fmt.set_parse_integer_only(True)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 54, reason="ICU4C<54")
def test_api_54():
    from icupy import UCurrencyUsage

    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # UCurrencyUsage icu::DecimalFormat::getCurrencyUsage()
    assert fmt.get_currency_usage() == UCurrencyUsage.UCURR_USAGE_STANDARD

    # void icu::DecimalFormat::setCurrencyUsage(
    #       UCurrencyUsage newUsage,
    #       UErrorCode *ec
    # )
    fmt.set_currency_usage(UCurrencyUsage.UCURR_USAGE_CASH)
    assert fmt.get_currency_usage() == UCurrencyUsage.UCURR_USAGE_CASH

    # UBool icu::DecimalFormat::isDecimalPatternMatchRequired(void)
    assert not fmt.is_decimal_pattern_match_required()

    # void icu::DecimalFormat::setDecimalPatternMatchRequired(UBool newValue)
    fmt.set_decimal_pattern_match_required(True)
    assert fmt.is_decimal_pattern_match_required()


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 62, reason="ICU4C<62")
def test_api_62():
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # int32_t icu::DecimalFormat::getMultiplierScale(void)
    assert fmt.get_multiplier_scale() == 0

    # void icu::DecimalFormat::setMultiplierScale(int32_t newValue)
    fmt.set_multiplier_scale(2)
    assert fmt.get_multiplier_scale() == 2


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 64, reason="ICU4C<64")
def test_api_64():
    from icupy.number import LocalizedNumberFormatter

    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("0", symbols)

    # int32_t icu::DecimalFormat::getMinimumGroupingDigits()
    assert fmt.get_minimum_grouping_digits() == -1

    # void icu::DecimalFormat::setMinimumGroupingDigits(int32_t newValue)
    fmt.set_minimum_grouping_digits(1)
    assert fmt.get_minimum_grouping_digits() == 1

    # UBool icu::DecimalFormat::isFormatFailIfMoreThanMaxDigits()
    assert not fmt.is_format_fail_if_more_than_max_digits()

    # void icu::DecimalFormat::setFormatFailIfMoreThanMaxDigits(UBool value)
    fmt.set_format_fail_if_more_than_max_digits(True)
    assert fmt.is_format_fail_if_more_than_max_digits()

    # UBool icu::DecimalFormat::isParseCaseSensitive()
    assert not fmt.is_parse_case_sensitive()

    # void icu::DecimalFormat::setParseCaseSensitive(UBool value)
    fmt.set_parse_case_sensitive(True)
    assert fmt.is_parse_case_sensitive()

    # UBool icu::DecimalFormat::isParseNoExponent()
    assert not fmt.is_parse_no_exponent()

    # void icu::DecimalFormat::setParseNoExponent(UBool value)
    fmt.set_parse_no_exponent(True)
    assert fmt.is_parse_no_exponent()

    # UBool icu::DecimalFormat::isSignAlwaysShown()
    assert not fmt.is_sign_always_shown()

    # void icu::DecimalFormat::setSignAlwaysShown(UBool value)
    fmt.set_sign_always_shown(True)
    assert fmt.is_sign_always_shown()

    # const number::LocalizedNumberFormatter *
    # icu::DecimalFormat::toNumberFormatter(UErrorCode &status)
    result = fmt.to_number_formatter()
    assert isinstance(result, LocalizedNumberFormatter)


def test_apply_localized_pattern():
    fmt = DecimalFormat()

    # [1]
    # void icu::DecimalFormat::applyLocalizedPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt.apply_localized_pattern(UnicodeString("0"))

    # UnicodeString &icu::DecimalFormat::toLocalizedPattern(
    #       UnicodeString &result
    # )
    result = UnicodeString()
    output = fmt.to_localized_pattern(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert result == "0"

    fmt.apply_localized_pattern("#,##0.#")
    output = fmt.to_localized_pattern(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert result == "#,##0.#"

    # [2]
    # void icu::DecimalFormat::applyLocalizedPattern(
    #       const UnicodeString &pattern,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    parse_error = UParseError()
    fmt.apply_localized_pattern(UnicodeString("0"), parse_error)
    assert parse_error.line <= 0
    assert parse_error.offset == 0

    parse_error = UParseError()
    fmt.apply_localized_pattern("0", parse_error)
    assert parse_error.line <= 0
    assert parse_error.offset == 0


def test_apply_pattern():
    fmt = DecimalFormat()

    # [1]
    # void icu::DecimalFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt.apply_pattern(UnicodeString("0"))

    # UnicodeString &icu::DecimalFormat::toPattern(UnicodeString &result)
    result = UnicodeString()
    output = fmt.to_pattern(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert result == "0"

    fmt.apply_pattern("#,##0.#")
    output = fmt.to_pattern(result)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(output)
    assert result == "#,##0.#"

    # [2]
    # void icu::DecimalFormat::applyPattern(
    #       const UnicodeString &pattern,
    #       UParseError &parseError,
    #       UErrorCode &status
    # )
    parse_error = UParseError()
    fmt.apply_pattern(UnicodeString("0"), parse_error)
    assert parse_error.line <= 0
    assert parse_error.offset == 0

    parse_error = UParseError()
    fmt.apply_pattern("0", parse_error)
    assert parse_error.line <= 0
    assert parse_error.offset == 0


def test_clone():
    fmt1 = DecimalFormat()

    # DecimalFormat *icu::DecimalFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, DecimalFormat)
    assert fmt1 == fmt2

    fmt3 = copy.copy(fmt1)
    assert fmt1 == fmt3

    fmt4 = copy.deepcopy(fmt1)
    assert fmt1 == fmt4


def test_decimal_format():
    assert issubclass(DecimalFormat, NumberFormat)

    # [1]
    # icu::DecimalFormat::DecimalFormat(UErrorCode &status)
    fmt1 = DecimalFormat()

    # [2]
    # icu::DecimalFormat::DecimalFormat(
    #       const UnicodeString &pattern,
    #       UErrorCode &status
    # )
    fmt2 = DecimalFormat(UnicodeString("0"))
    assert fmt1 != fmt2

    fmt2a = DecimalFormat("0")
    assert fmt2 == fmt2a

    # [6]
    # icu::DecimalFormat::DecimalFormat(
    #       const UnicodeString &pattern,
    #       const DecimalFormatSymbols &symbols,
    #       UErrorCode &status
    # )
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt6 = DecimalFormat(UnicodeString("0"), symbols)

    fmt6a = DecimalFormat("0", symbols)
    assert fmt6 == fmt6a

    # [7]
    # icu::DecimalFormat::DecimalFormat(const DecimalFormat &source)
    fmt7 = DecimalFormat(fmt1)
    assert fmt1 == fmt7


def test_format():
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("#,##0.#", symbols)
    d = -10456.0037
    s = "-1.045600e+04"
    n32 = INT32_MAX
    n64 = INT32_MAX + 1
    append_to = UnicodeString()

    # [1]
    # UnicodeString &icu::NumberFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(Formattable(d), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # [2]
    # UnicodeString &icu::NumberFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    result = fmt.format(Formattable(d), append_to, pos_iter)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    append_to.remove()
    result = fmt.format(Formattable(d), append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # [7]
    # UnicodeString &icu::NumberFormat::format(
    #       double number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(d, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # [8], [9]
    # UnicodeString &icu::DecimalFormat::format(
    #       double number,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(d, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # [12], [13]
    # UnicodeString &icu::NumberFormat::format(
    #       double number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    result = fmt.format(d, append_to, pos_iter)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    append_to.remove()
    result = fmt.format(d, append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # [14]
    # UnicodeString &icu::NumberFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(n32, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,647"

    # [15], [16]
    # UnicodeString &icu::DecimalFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n32, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,647"

    # [19], [20]
    # UnicodeString &icu::NumberFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    result = fmt.format(n32, append_to, pos_iter)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,647"

    append_to.remove()
    result = fmt.format(n32, append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,647"

    # [21]
    # UnicodeString &icu::NumberFormat::format(
    #       int64_t number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(n64, append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,648"

    # [22], [23]
    # UnicodeString &icu::NumberFormat::format(
    #       int64_t number,
    #       UnicodeString &appendTo
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n64, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,648"

    # [26], [27]
    # UnicodeString &icu::NumberFormat::format(
    #       int64_t number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    result = fmt.format(n64, append_to, pos_iter)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,648"

    append_to.remove()
    result = fmt.format(n64, append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "2,147,483,648"

    # [28], [29]
    # UnicodeString &icu::NumberFormat::format(
    #       StringPiece number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    result = fmt.format(s, append_to, pos_iter)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    append_to.remove()
    result = fmt.format(s, append_to, None)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"

    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    append_to.remove()
    result = fmt.format(Formattable(d), append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "-10,456"


def test_parse():
    symbols = DecimalFormatSymbols(Locale.get_us())
    fmt = DecimalFormat("#,##0.#", symbols)
    d = -10456.0037
    s = "-10,456.0037"

    # [1], [2]
    # void icu::DecimalFormat::parse(
    #       const UnicodeString &text,
    #       Formattable &result,
    #       ParsePosition &parsePosition
    # )
    result = Formattable()
    parse_position = ParsePosition()
    fmt.parse(UnicodeString(s), result, parse_position)
    assert parse_position.get_error_index() == -1
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    result = Formattable()
    parse_position = ParsePosition()
    fmt.parse(s, result, parse_position)
    assert parse_position.get_error_index() == -1
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    # [3]
    # void icu::DecimalFormat::parse(
    #       const UnicodeString &text,
    #       Formattable &result,
    #       UErrorCode &status
    # )
    result = Formattable()
    fmt.parse(UnicodeString(s), result)
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    result = Formattable()
    fmt.parse(s, result)
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d



import pytest
from icupy import (
    AlphabeticIndex, ConstVoidPtr, Locale, RuleBasedCollator,
    UAlphabeticIndexLabelType, U_ICU_VERSION_MAJOR_NUM, UnicodeSet,
    UnicodeString,
)


def test_add_labels():
    # [1]
    # AlphabeticIndex &icu::AlphabeticIndex::addLabels(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    index = AlphabeticIndex(Locale("en"))
    result = index.add_labels(Locale("ja", "JP"))
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_bucket_count() > 35

    index = AlphabeticIndex(Locale("en"))
    result = index.add_labels("ja_JP")
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_bucket_count() > 35

    # [2]
    # AlphabeticIndex &icu::AlphabeticIndex::addLabels(
    #       const UnicodeSet &additions,
    #       UErrorCode &status
    # )
    index = AlphabeticIndex(Locale("en"))
    additions = UnicodeSet()
    additions.add(0x410).add(0x415)
    result = index.add_labels(additions)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    # underflow, A-Z, inflow, 2 Cyrillic, overflow
    assert index.get_bucket_count() == 31


def test_add_record():
    index = AlphabeticIndex(Locale.get_english())
    obj3 = index
    obj4 = {"AlphabeticIndex": index}

    # UBool icu::AlphabeticIndex::nextBucket(UErrorCode &status)
    assert index.next_bucket()

    # UBool icu::AlphabeticIndex::nextRecord(UErrorCode &status)
    assert not index.next_record()

    # AlphabeticIndex &icu::AlphabeticIndex::addRecord(
    #       const UnicodeString &name,
    #       const void *data,
    #       UErrorCode &status
    # )
    result = index.add_record(UnicodeString("Adam"), None)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    result = index.add_record("Baker", None)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    data3 = ConstVoidPtr(obj3)
    result = index.add_record(UnicodeString("Charlie"), data3)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    data4 = ConstVoidPtr(obj4)
    result = index.add_record("Chad", data4)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    # int32_t icu::AlphabeticIndex::getRecordCount(UErrorCode &status)
    assert index.get_record_count() == 4

    # AlphabeticIndex &icu::AlphabeticIndex::resetBucketIterator(
    #       UErrorCode &status
    # )
    result = index.reset_bucket_iterator()
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    # int32_t icu::AlphabeticIndex::getBucketRecordCount()
    # const void *icu::AlphabeticIndex::getRecordData()
    # const UnicodeString &icu::AlphabeticIndex::getRecordName()
    assert index.next_bucket()
    assert index.get_bucket_record_count() == 0
    assert not index.next_record()

    assert index.next_bucket()
    assert index.get_bucket_record_count() == 1
    assert index.next_record()
    result = index.get_record_data()
    assert result is None
    result = index.get_record_name()
    assert isinstance(result, UnicodeString)
    assert result == "Adam"
    assert not index.next_record()

    assert index.next_bucket()
    assert index.get_bucket_record_count() == 1
    assert index.next_record()
    result = index.get_record_data()
    assert result is None
    result = index.get_record_name()
    assert isinstance(result, UnicodeString)
    assert result == "Baker"
    assert not index.next_record()

    assert index.next_bucket()
    assert index.get_bucket_record_count() == 2
    assert index.next_record()
    result = index.get_record_data()
    assert isinstance(result, ConstVoidPtr)
    assert result.to_object() == obj4
    result = index.get_record_name()
    assert isinstance(result, UnicodeString)
    assert result == "Chad"

    assert index.next_record()
    result = index.get_record_data()
    assert isinstance(result, ConstVoidPtr)
    assert result.to_object() == obj3
    result = index.get_record_name()
    assert isinstance(result, UnicodeString)
    assert result == "Charlie"
    assert not index.next_record()

    # AlphabeticIndex &icu::AlphabeticIndex::resetRecordIterator()
    result = index.reset_record_iterator()
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)

    assert index.next_record()
    assert index.get_record_name() == "Chad"

    # AlphabeticIndex &icu::AlphabeticIndex::clearRecords(UErrorCode &status)
    result = index.clear_records()
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_record_count() == 0


def test_api():
    # [1]
    # icu::AlphabeticIndex::AlphabeticIndex(
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    index = AlphabeticIndex(Locale("en"))
    index2 = AlphabeticIndex("en")

    # int32_t icu::AlphabeticIndex::getBucketCount(UErrorCode &status)
    assert index.get_bucket_count() == 28  # A-Z, underflow, overflow
    assert index2.get_bucket_count() == 28

    # UBool icu::AlphabeticIndex::nextBucket(UErrorCode &status)
    assert index.next_bucket()
    assert index.next_bucket()
    assert index.next_bucket()

    # [1]
    # int32_t icu::AlphabeticIndex::getBucketIndex()
    assert index.get_bucket_index() == 2

    # [2]
    # int32_t icu::AlphabeticIndex::getBucketIndex(
    #       const UnicodeString &itemName,
    #       UErrorCode &status
    # )
    assert index.get_bucket_index(UnicodeString("A")) == 1
    assert index.get_bucket_index("B") == 2

    # const UnicodeString &icu::AlphabeticIndex::getBucketLabel()
    result = index.get_bucket_label()
    assert isinstance(result, UnicodeString)
    assert result == "B"

    # UAlphabeticIndexLabelType icu::AlphabeticIndex::getBucketLabelType()
    assert (index.get_bucket_label_type()
            == UAlphabeticIndexLabelType.U_ALPHAINDEX_NORMAL)

    # const RuleBasedCollator &icu::AlphabeticIndex::getCollator()
    result = index.get_collator()
    assert isinstance(result, RuleBasedCollator)

    # const UnicodeString &icu::AlphabeticIndex::getInflowLabel()
    result = index.get_inflow_label()
    assert isinstance(result, UnicodeString)
    assert result == "\u2026"  # "…"

    # int32_t icu::AlphabeticIndex::getMaxLabelCount()
    assert index.get_max_label_count() == 99

    # const UnicodeString &icu::AlphabeticIndex::getOverflowLabel()
    result = index.get_overflow_label()
    assert isinstance(result, UnicodeString)
    assert result == "\u2026"  # "…"

    # const UnicodeString &icu::AlphabeticIndex::getUnderflowLabel()
    result = index.get_underflow_label()
    assert isinstance(result, UnicodeString)
    assert result == "\u2026"  # "…"

    # AlphabeticIndex &icu::AlphabeticIndex::resetBucketIterator(
    #       UErrorCode &status
    # )
    result = index.reset_bucket_iterator()
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.next_bucket()
    assert index.get_bucket_index() == 0

    # AlphabeticIndex &icu::AlphabeticIndex::setInflowLabel(
    #       const UnicodeString &inflowLabel,
    #       UErrorCode &status
    # )
    result = index.set_inflow_label(UnicodeString("I"))
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_inflow_label() == "I"

    result = index.set_inflow_label("i")
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_inflow_label() == "i"

    # AlphabeticIndex &icu::AlphabeticIndex::setMaxLabelCount(
    #       int32_t maxLabelCount,
    #       UErrorCode &status
    # )
    result = index.set_max_label_count(256)
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_max_label_count() == 256

    # AlphabeticIndex &icu::AlphabeticIndex::setOverflowLabel(
    #       const UnicodeString &overflowLabel,
    #       UErrorCode &status
    # )
    result = index.set_overflow_label(UnicodeString("O"))
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_overflow_label() == "O"

    result = index.set_overflow_label("o")
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_overflow_label() == "o"

    # AlphabeticIndex &icu::AlphabeticIndex::setUnderflowLabel(
    #       const UnicodeString &underflowLabel,
    #       UErrorCode &status
    # )
    result = index.set_underflow_label(UnicodeString("U"))
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_underflow_label() == "U"

    result = index.set_underflow_label("u")
    assert isinstance(result, AlphabeticIndex)
    assert id(result) == id(index)
    assert index.get_underflow_label() == "u"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 51, reason="ICU4C<51")
def test_api_51():
    from icupy import Collator

    # [2]
    # icu::AlphabeticIndex::AlphabeticIndex(
    #       RuleBasedCollator *collator,
    #       UErrorCode &status
    # )
    collator = Collator.create_instance(Locale.get_english())
    index = AlphabeticIndex(collator)
    index.add_labels(Locale.get_english())
    assert index.get_bucket_count() == 28

    # const RuleBasedCollator &icu::AlphabeticIndex::getCollator()
    result = index.get_collator()
    assert isinstance(result, RuleBasedCollator)
    assert result == collator

    # ImmutableIndex *icu::AlphabeticIndex::buildImmutableIndex(
    #       UErrorCode &errorCode
    # )
    imm_index = index.build_immutable_index()
    assert isinstance(imm_index, AlphabeticIndex.ImmutableIndex)

    # const Bucket *icu::AlphabeticIndex::ImmutableIndex::getBucket(
    #       int32_t index
    # )
    bucket = imm_index.get_bucket(1)
    assert isinstance(bucket, AlphabeticIndex.Bucket)

    # int32_t icu::AlphabeticIndex::ImmutableIndex::getBucketCount()
    assert imm_index.get_bucket_count() == 28

    # int32_t icu::AlphabeticIndex::ImmutableIndex::getBucketIndex(
    #       const UnicodeString &name,
    #       UErrorCode &errorCode
    # )
    assert imm_index.get_bucket_index(UnicodeString("A")) == 1
    assert imm_index.get_bucket_index("B") == 2

    # const UnicodeString &icu::AlphabeticIndex::Bucket::getLabel()
    result = bucket.get_label()
    assert isinstance(result, UnicodeString)
    assert result == "A"

    # UAlphabeticIndexLabelType icu::AlphabeticIndex::Bucket::getLabelType()
    assert (bucket.get_label_type()
            == UAlphabeticIndexLabelType.U_ALPHAINDEX_NORMAL)



import copy

import pytest
from icupy import (
    FieldPosition, FieldPositionIterator, Formattable, ICUError,
    INT32_MAX, INT64_MAX, Locale, NumberFormat, ParsePosition,
    RuleBasedNumberFormat, UErrorCode, UParseError, URBNFRuleSetTag,
    U_ICU_VERSION_MAJOR_NUM, UnicodeString,
)


def test_api():
    # From icu/source/test/intltest/itrbnf.cpp
    rules = (
        "%main:"
        " 0: no; 1: some; 100: a lot; 1000: tons;\n"
        "%other:"
        " 0: nada; 1: yah, some; 100: plenty; 1000: more'n you'll ever need"
    )
    localizations = (
        "<"
        " <%main, %other>, <en, Main, Other>, <fr, leMain, leOther>,"
        " <de, 'das Main', 'etwas anderes'>"
        ">"
    )
    perror = UParseError()
    fmt = RuleBasedNumberFormat(rules, localizations, perror)

    # UnicodeString icu::RuleBasedNumberFormat::getDefaultRuleSetName()
    result = fmt.get_default_rule_set_name()
    assert isinstance(result, UnicodeString)
    assert result == "%main"

    # void icu::RuleBasedNumberFormat::setDefaultRuleSet(
    #       const UnicodeString &ruleSetName,
    #       UErrorCode &status
    # )
    fmt.set_default_rule_set(UnicodeString("%other"))
    assert fmt.get_default_rule_set_name() == "%other"

    fmt.set_default_rule_set("%main")
    assert fmt.get_default_rule_set_name() == "%main"

    # int32_t icu::RuleBasedNumberFormat::getNumberOfRuleSetDisplayNameLocales(
    #       void
    # )
    assert fmt.get_number_of_rule_set_display_name_locales() == 3

    # int32_t icu::RuleBasedNumberFormat::getNumberOfRuleSetNames()
    assert fmt.get_number_of_rule_set_names() == 2

    # UnicodeString icu::RuleBasedNumberFormat::getRules()
    result = fmt.get_rules()
    assert isinstance(result, UnicodeString)
    assert len(result) > 0

    # Locale icu::RuleBasedNumberFormat::getRuleSetDisplayNameLocale(
    #       int32_t index,
    #       UErrorCode &status
    # )
    result = fmt.get_rule_set_display_name_locale(1)
    assert isinstance(result, Locale)
    assert result == Locale("fr")

    # UnicodeString icu::RuleBasedNumberFormat::getRuleSetName(int32_t index)
    result = fmt.get_rule_set_name(1)
    assert isinstance(result, UnicodeString)
    assert result == "%other"

    # UBool icu::RuleBasedNumberFormat::isLenient(void)
    assert not fmt.is_lenient()

    # void icu::RuleBasedNumberFormat::setLenient(UBool enabled)
    fmt.set_lenient(True)
    assert fmt.is_lenient()

    # UBool icu::RuleBasedNumberFormat::operator==(const Format &other)
    fmt2 = fmt.clone()
    fmt3 = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())
    assert fmt == fmt2
    assert not (fmt == fmt3)
    assert not (fmt2 == fmt3)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 49, reason="ICU4C<49")
def test_api_49():
    from icupy import DecimalFormatSymbols

    fmt = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_ORDINAL,
        Locale.get_us())

    # void icu::RuleBasedNumberFormat::setDecimalFormatSymbols(
    #       const DecimalFormatSymbols &symbols
    # )
    symbols = DecimalFormatSymbols(Locale.get_us())
    symbols.set_symbol(
        DecimalFormatSymbols.GROUPING_SEPARATOR_SYMBOL,
        UnicodeString("&"),
        True)
    fmt.set_decimal_format_symbols(symbols)
    append_to = UnicodeString()
    assert fmt.format(1001, append_to) == "1&001st"


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 53, reason="ICU4C<53")
def test_api_53():
    from icupy import UDisplayContext, UDisplayContextType

    fmt = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())

    # UDisplayContext icu::NumberFormat::getContext(
    #       UDisplayContextType type,
    #       UErrorCode &status
    # )
    assert (fmt.get_context(UDisplayContextType.UDISPCTX_TYPE_CAPITALIZATION)
            == UDisplayContext.UDISPCTX_CAPITALIZATION_NONE)

    # void icu::RuleBasedNumberFormat::setContext(
    #       UDisplayContext value,
    #       UErrorCode &status
    # )
    fmt.set_context(UDisplayContext.UDISPCTX_CAPITALIZATION_FOR_STANDALONE)
    assert (fmt.get_context(UDisplayContextType.UDISPCTX_TYPE_CAPITALIZATION)
            == UDisplayContext.UDISPCTX_CAPITALIZATION_FOR_STANDALONE)


@pytest.mark.skipif(U_ICU_VERSION_MAJOR_NUM < 60, reason="ICU4C<60")
def test_api_60():
    fmt = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())

    # ERoundingMode icu::RuleBasedNumberFormat::getRoundingMode(void)
    assert fmt.get_rounding_mode() == RuleBasedNumberFormat.ROUND_UNNECESSARY

    # void icu::RuleBasedNumberFormat::setRoundingMode(
    #       ERoundingMode roundingMode
    # )
    fmt.set_rounding_mode(RuleBasedNumberFormat.ROUND_HALF_EVEN)
    assert fmt.get_rounding_mode() == RuleBasedNumberFormat.ROUND_HALF_EVEN


def test_clone():
    fmt1 = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())

    # RuleBasedNumberFormat *icu::RuleBasedNumberFormat::clone()
    fmt2 = fmt1.clone()
    assert isinstance(fmt2, RuleBasedNumberFormat)
    assert fmt1 == fmt2

    fmt3 = copy.copy(fmt1)
    assert fmt1 == fmt3

    fmt4 = copy.deepcopy(fmt1)
    assert fmt1 == fmt4


def test_format():
    fmt = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())
    d = -10456.0037
    s = "-1.045600e+04"
    n32 = INT32_MAX
    n64 = INT64_MAX
    append_to = UnicodeString()
    rule_set_name = UnicodeString("%spellout-numbering")

    # [1], [2]
    # UnicodeString &icu::NumberFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(Formattable(d), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [3], [4]
    # UnicodeString &icu::NumberFormat::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(Formattable(d), append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [5]
    # UnicodeString &icu::Format::format(
    #       const Formattable &obj,
    #       UnicodeString &appendTo,
    #       UErrorCode &status
    # )
    append_to.remove()
    result = fmt.format(Formattable(d), append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    # [9]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       double number,
    #       const UnicodeString &ruleSetName,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(d, rule_set_name, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(d, str(rule_set_name), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    # [10]
    # UnicodeString &icu::NumberFormat::format(
    #       double number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(d, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    # [11], [14]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       double number,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(d, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("minus ten thousand four hundred fifty-six"
                      " point zero zero three seven")

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [13]
    # UnicodeString &icu::NumberFormat::format(
    #       double number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(d, append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [15]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       int32_t number,
    #       const UnicodeString &ruleSetName,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n32, rule_set_name, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("two billion one hundred forty-seven million"
                      " four hundred eighty-three thousand"
                      " six hundred forty-seven")

    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n32, str(rule_set_name), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("two billion one hundred forty-seven million"
                      " four hundred eighty-three thousand"
                      " six hundred forty-seven")

    # [16]
    # UnicodeString &icu::NumberFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(n32, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("two billion one hundred forty-seven million"
                      " four hundred eighty-three thousand"
                      " six hundred forty-seven")

    # [17], [20]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       int32_t number,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n32, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == ("two billion one hundred forty-seven million"
                      " four hundred eighty-three thousand"
                      " six hundred forty-seven")

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [19]
    # UnicodeString &icu::NumberFormat::format(
    #       int32_t number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(n32, append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # [21]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       int64_t number,
    #       const UnicodeString &ruleSetName,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n64, rule_set_name, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "9,223,372,036,854,775,807"

    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n64, str(rule_set_name), append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "9,223,372,036,854,775,807"

    # [22]
    # UnicodeString &icu::NumberFormat::format(
    #       int64_t number,
    #       UnicodeString &appendTo
    # )
    append_to.remove()
    result = fmt.format(n64, append_to)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "9,223,372,036,854,775,807"

    # [23], [26]
    # UnicodeString &icu::RuleBasedNumberFormat::format(
    #       int64_t number,
    #       UnicodeString &toAppendTo,
    #       FieldPosition &pos
    # )
    append_to.remove()
    pos = FieldPosition(FieldPosition.DONT_CARE)
    result = fmt.format(n64, append_to, pos)
    assert isinstance(result, UnicodeString)
    assert id(result) == id(append_to)
    assert result == "9,223,372,036,854,775,807"

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [25]
    # UnicodeString &format(
    #       int64_t number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(n64, append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR

    # *U_UNSUPPORTED_ERROR in ICU 69*
    # [27]
    # UnicodeString &icu::NumberFormat::format(
    #       StringPiece number,
    #       UnicodeString &appendTo,
    #       FieldPositionIterator *posIter,
    #       UErrorCode &status
    # )
    append_to.remove()
    pos_iter = FieldPositionIterator()
    with pytest.raises(ICUError) as exc_info:
        _ = fmt.format(s, append_to, pos_iter)
    assert exc_info.value.args[0] == UErrorCode.U_UNSUPPORTED_ERROR


def test_get_rule_set_display_name():
    # From icu/source/test/intltest/itrbnf.cpp
    rules = (
        "%main:"
        " 0: no; 1: some; 100: a lot; 1000: tons;\n"
        "%other:"
        " 0: nada; 1: yah, some; 100: plenty; 1000: more'n you'll ever need"
    )
    localizations = (
        "<"
        " <%main, %other>, <en, Main, Other>, <fr, leMain, leOther>,"
        " <de, 'das Main', 'etwas anderes'>"
        ">"
    )
    perror = UParseError()
    fmt = RuleBasedNumberFormat(rules, localizations, perror)

    # [1]
    # UnicodeString icu::RuleBasedNumberFormat::getRuleSetDisplayName(
    #       const UnicodeString &ruleSetName,
    #       const Locale &locale = Locale::getDefault()
    # )
    result = fmt.get_rule_set_display_name(UnicodeString("%main"))
    assert isinstance(result, UnicodeString)
    assert result in ["%main", "Main", "leMain", "das Main"]

    result = fmt.get_rule_set_display_name("%main")
    assert isinstance(result, UnicodeString)
    assert result in ["%main", "Main", "leMain", "das Main"]

    result = fmt.get_rule_set_display_name(
        UnicodeString("%main"),
        Locale("de_DE_FOO"))
    assert isinstance(result, UnicodeString)
    assert result == "das Main"

    result = fmt.get_rule_set_display_name(
        "%main",
        Locale("de_DE_FOO"))
    assert isinstance(result, UnicodeString)
    assert result == "das Main"

    result = fmt.get_rule_set_display_name(
        UnicodeString("%main"),
        "de_DE_FOO")
    assert isinstance(result, UnicodeString)
    assert result == "das Main"

    result = fmt.get_rule_set_display_name(
        "%main",
        "de_DE_FOO")
    assert isinstance(result, UnicodeString)
    assert result == "das Main"

    # [2]
    # UnicodeString icu::RuleBasedNumberFormat::getRuleSetDisplayName(
    #       int32_t index,
    #       const Locale &locale = Locale::getDefault()
    # )
    result = fmt.get_rule_set_display_name(0)
    assert isinstance(result, UnicodeString)
    assert result in ["%main", "Main", "leMain", "das Main"]

    result = fmt.get_rule_set_display_name(0, Locale("de_DE_FOO"))
    assert isinstance(result, UnicodeString)
    assert result == "das Main"

    result = fmt.get_rule_set_display_name(0, "de_DE_FOO")
    assert isinstance(result, UnicodeString)
    assert result == "das Main"


def test_parse():
    fmt = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())
    d = -10456.0037
    s = ("minus ten thousand four hundred fifty-six"
         " point zero zero three seven")

    # [1], [2]
    # void icu::RuleBasedNumberFormat::parse(
    #       const UnicodeString &text,
    #       Formattable &result,
    #       ParsePosition &parsePosition
    # )
    result = Formattable()
    parse_position = ParsePosition()
    fmt.parse(UnicodeString(s), result, parse_position)
    assert parse_position.get_index() > 0
    assert parse_position.get_error_index() == -1
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    result = Formattable()
    parse_position = ParsePosition()
    fmt.parse(s, result, parse_position)
    assert parse_position.get_index() > 0
    assert parse_position.get_error_index() == -1
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    # [3]
    # void icu::NumberFormat::parse(
    #       const UnicodeString &text,
    #       Formattable &result,
    #       UErrorCode &status
    # )
    result = Formattable()
    fmt.parse(UnicodeString(s), result)
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d

    result = Formattable()
    fmt.parse(s, result)
    assert result.get_type() == Formattable.DOUBLE
    assert result.get_double() == d


def test_rule_based_number_format():
    assert issubclass(RuleBasedNumberFormat, NumberFormat)

    rules = UnicodeString("=#,##0=;1,000,000: <##0.###< million;")

    # [1]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       const UnicodeString &rules,
    #       UParseError &perror,
    #       UErrorCode &status
    # )
    perror = UParseError()
    fmt1 = RuleBasedNumberFormat(rules, perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt1a = RuleBasedNumberFormat(str(rules), perror)
    assert perror.offset == 0
    assert fmt1 == fmt1a

    # [2]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       const UnicodeString &rules,
    #       const UnicodeString &localizations,
    #       UParseError &perror,
    #       UErrorCode &status
    # )
    perror = UParseError()
    fmt2 = RuleBasedNumberFormat(rules, UnicodeString(), perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt2a = RuleBasedNumberFormat(str(rules), UnicodeString(), perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt2b = RuleBasedNumberFormat(rules, "", perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt2c = RuleBasedNumberFormat(str(rules), "", perror)
    assert perror.offset == 0
    assert fmt2 == fmt2a == fmt2b == fmt2c

    # [3]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       const UnicodeString &rules,
    #       const Locale &locale,
    #       UParseError &perror,
    #       UErrorCode &status
    # )
    perror = UParseError()
    fmt3 = RuleBasedNumberFormat(
        rules,
        Locale.get_default(),
        perror)
    assert perror.offset == 0
    assert fmt1 == fmt3

    perror = UParseError()
    fmt3a = RuleBasedNumberFormat(
        str(rules),
        Locale.get_default(),
        perror)
    assert perror.offset == 0
    assert fmt3 == fmt3a

    # [4]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       const UnicodeString &rules,
    #       const UnicodeString &localizations,
    #       const Locale &locale,
    #       UParseError &perror,
    #       UErrorCode &status
    # )
    perror = UParseError()
    fmt4 = RuleBasedNumberFormat(
        rules,
        UnicodeString(),
        Locale.get_default(),
        perror)
    assert perror.offset == 0
    assert fmt2 == fmt4

    perror = UParseError()
    fmt4a = RuleBasedNumberFormat(
        str(rules),
        UnicodeString(),
        Locale.get_default(),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4b = RuleBasedNumberFormat(
        rules,
        "",
        Locale.get_default(),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4c = RuleBasedNumberFormat(
        rules,
        UnicodeString(),
        str(Locale.get_default()),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4d = RuleBasedNumberFormat(
        str(rules),
        "",
        Locale.get_default(),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4e = RuleBasedNumberFormat(
        rules,
        "",
        str(Locale.get_default()),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4f = RuleBasedNumberFormat(
        str(rules),
        UnicodeString(),
        str(Locale.get_default()),
        perror)
    assert perror.offset == 0

    perror = UParseError()
    fmt4g = RuleBasedNumberFormat(
        str(rules),
        "",
        str(Locale.get_default()),
        perror)
    assert perror.offset == 0
    assert fmt4 == fmt4a == fmt4b == fmt4c == fmt4d == fmt4e == fmt4f == fmt4g

    # [5]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       URBNFRuleSetTag tag,
    #       const Locale &locale,
    #       UErrorCode &status
    # )
    fmt5 = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        Locale.get_us())

    fmt5a = RuleBasedNumberFormat(
        URBNFRuleSetTag.URBNF_SPELLOUT,
        "en-US")
    assert fmt5 == fmt5a

    # [6]
    # icu::RuleBasedNumberFormat::RuleBasedNumberFormat(
    #       const RuleBasedNumberFormat &rhs
    # )
    fmt6 = RuleBasedNumberFormat(fmt5)
    assert fmt5 == fmt6
